# frozen_string_literal: true

module MergeRequests
  # Service for manually merged Mercurial Merge Requests
  #
  # - update the MR to the latest relevant changeset (can be `nil` if caller
  #   cannot find it)
  # - mark it as merged
  # - execute all hooks and notifications
  #
  class HgManuallyMergedService < MergeRequests::BaseService
    def execute(merge_request, latest_sha)
      refresh(merge_request, latest_sha)
      MergeRequests::PostMergeService.new(project: merge_request.target_project, current_user: @current_user).execute(merge_request)
    end

    # Refresh the MR to include a new commit, without reloading everything.
    #
    # Typically, with topics publication, the source branch has already vanished
    # reloading everything would not understand that.
    def refresh(merge_request, real_source_sha)
      return if real_source_sha.nil?

      recorded_source_sha = merge_request.diff_head_sha
      return if recorded_source_sha == real_source_sha

      # the new diff creation apparently makes several Gitaly calls,
      # which can be an error depending on running options (probably not in
      # production, but that may change in GitLab versions). It's not the
      # responsibility of this service to avoid that.
      Gitlab::GitalyClient.allow_n_plus_1_calls do
        old_diff_refs = merge_request.diff_refs
        # base should not have changed if target is a Mercurial named
        # branch (TODO one day we'll have MRs between topics)

        MergeRequestDiff.create(
          merge_request_id: merge_request.id,
          head_commit_sha: real_source_sha,
          base_commit_sha: old_diff_refs.base_sha,
          start_commit_sha: old_diff_refs.start_sha
        )

        new_diff_refs = merge_request.diff_refs
        merge_request.update_diff_discussion_positions(
          old_diff_refs: old_diff_refs,
          new_diff_refs: new_diff_refs,
          current_user: @current_user)
      end
    end
  end
end
