# frozen_string_literal: true

module Gitlab
  module Mercurial
    class HgitalyRepository < HgGitRepository
      extend ::Gitlab::Utils::Override
      FULL_SHA_PATTERN = /\A\h{40}\z/.freeze

      attr_writer :is_hgitaly1

      def initialize(*args, hpd_original: nil)
        super(*args)
        @hpd_original = hpd_original
      end

      override :hgitaly1?
      def hgitaly1?
        @is_hgitaly1
      end

      # Return `true` if we have an auxiliary Git repository to maintain
      # with hg-git, as in HgGitRepository.
      #
      # Should eventually come from a Project setting, when fully native mode
      # becomes default, even though it won't be needed by Heptapod itself,
      # see heptapod#125
      def hg_with_git?
        true
      end

      override :hg_fully_native?
      def hg_fully_native?
        !@is_hgitaly1
      end

      def hpd_original
        @hpd_original.nil? ? self : @hpd_original
      end

      def hg_sha_map
        @hpd_original.nil? ? super : @hpd_original.hg_sha_map
      end

      def git_sha_map
        @hpd_original.nil? ? super : @hpd_original.git_sha_map
      end

      def cached_exists?
        @hpd_original.nil? ? super : @hpd_original.cached_exists?
      end

      override :archive_sha
      def archive_sha(sha)
        sha
      end

      def create_repository(**named_args)
        if hg_with_git?
          super(**named_args)
        else
          # do not create any Git repository
          create_hg_repository(**named_args)
        end
      end

      override :remove
      def remove
        if hg_with_git?
          super
        else
          # no Git repository to remove (leftovers from hgitaly1 times
          # will be considered independently).
          hg_remove
        end
      end

      override :size
      def size
        if hg_with_git?
          super
        else
          # there is no Git repository, or it's a leftover from hgitaly1
          # times, a problem of system admins, not of customers.
          hg_size
        end
      end

      override :bundle_to_disk
      def bundle_to_disk(save_path)
        if hg_with_git?
          super(save_path)
        else
          # there is no Git repository. We don't backup leftovers from
          # hgitaly1 times.
          hg_bundle_to_disk(save_path)
        end
      end

      override :create_from_bundle
      def create_from_bundle(path_to_bundle)
        if hg_with_git?
          super(path_to_bundle)
        else
          hg_create_from_bundle(path_to_bundle)
        end
      end

      # Compatibility scaffolding for uniformity with HgGitRepository
      # the SHAs we're handling in this class are directly Mercurial ones.
      # Of course, eventually we won't have HGitRepository around, and
      # we won't need the callers of this in Git::Commit and elsewhere
      def hgsha_from_sha(sha)
        sha
      end

      # Compatibility scaffolding: see hgsha_from_sha
      def sha_from_hgsha(hgsha)
        hgsha
      end

      # Compatibility scaffolding: see Repository#gitsha_from_sha
      def gitsha_from_sha(sha)
        return unless cached_exists?

        git_sha_map[sha]
      end

      # Compatibility scaffolding: see Repository#gitsha_from_sha
      def sha_from_gitsha(gitsha)
        return unless cached_exists?

        hg_sha_map[gitsha]
      end

      # Compatibility scaffolding, similar to hgsha_from_sha
      #
      # the difference here is that this method does not promise
      # to return a Mercurial SHA
      def commit_id_to_display_id(commit_id)
        commit_id
      end

      # See commit_id_to_display_id
      def commit_id_from_display_id(display_id)
        display_id
      end

      override :hg_env_minimal
      def hg_env_minimal(*args, **kwargs)
        res = super
        res['HEPTAPOD_HG_NATIVE'] = 'yes'
        res['HEPTAPOD_NO_GIT'] = (!hg_with_git?).to_s
        res
      end

      def list_last_commits_for_tree(sha, path, **kwargs)
        if hgitaly1?
          commits = super(gitsha_from_sha(sha), path, **kwargs)
          commits.each_value { |c| c.hpd_back_convert!(self) }
          commits
        else
          super
        end
      end

      def diff_stats(left_id, right_id)
        # incoming "ids" can be Git::EMPTY_TREE_ID or perhaps even
        # non-sha revisions
        if hgitaly1?
          # convert ids on the fly, for this through gitaly_commit_client
          # that actually invokes the DiffService

          super(fallbacking_gitsha_from_sha(left_id),
                fallbacking_gitsha_from_sha(right_id))
        else
          super
        end
      end

      # Except in fully native mode, ref writes must be done both
      # in the Mercurial (HGitaly, via super) and the auxiliary
      # Git repositories.
      #
      # Currently, HGitaly does not support old_ref, but this parameter
      # just checks that the previous value is as expected, so it is enough
      # to let Gitaly perform the check for the time being. See hgitaly#53 for
      # necessary improvements in the fully native mode.
      def write_ref(ref_path, ref, old_ref: nil, hg_only: false)
        if ref == Gitlab::Mercurial::NULL_NODE
          # can happen with before_shas in pipelines, but creates problems
          # in HGitaly and actually breaks functional tests.
          # Early return resolves this problem.
          return
        end

        if hg_with_git? && !hg_only
          git_target =
            if FULL_SHA_PATTERN.match(ref).nil?
              # not an exact hg sha, must be an actual ref, let it go
              # identically in both repos
              ref
            else
              # exact Mercurial sha must be converted to a Git sha
              gitsha_from_sha(ref)
            end

          git_repository.write_ref(
            ref_path,
            git_target,
            old_ref: old_ref.nil? ? nil : gitsha_from_sha(old_ref)
          )
        end

        super(ref_path, ref, old_ref: nil)
      end

      # See comment for #write_ref
      override :delete_refs
      def delete_refs(*ref_names)
        git_repository.delete_refs(*ref_names) if hg_with_git?
        super(*ref_names)
      end

      override :backup_additional
      def backup_additional(save_path)
        output, status = hg_call(
          ["hpd-ensure-all-gitlab-specific-state-files"],
          force_system_user: true)
        raise StandardError, output.strip unless status == 0

        super
      end

      # remove this override when upstream uses the Gitaly SetFullPath
      # independently of feature flags
      override :set_full_path
      def set_full_path(full_path:)
        return unless full_path.present?

        # This guard avoids Gitaly log/error spam
        raise NoRepository, 'repository does not exist' unless exists?

        gitaly_repository_client.set_full_path(full_path)
      end

      # Provides access to the auxiliary Git repository for special case.
      # Will be eventually removed, once all Mercurial native projects are
      # fully native with no feature flag
      def git_repository
        @git_repository ||= Gitlab::Git::Repository.new(@storage,
                                                        @relative_path,
                                                        @gl_repository,
                                                        @gl_project_path)
      end

      def gitaly_ref_client
        @gitaly_ref_client ||= Gitlab::GitalyClient::RefService.new(self, vcs: 'hg')
      end

      def gitaly_commit_client
        @gitaly_commit_client ||= Gitlab::GitalyClient::CommitService.new(self, vcs: 'hg')
      end

      def gitaly_repository_client
        @gitaly_repository_client ||= Gitlab::GitalyClient::RepositoryService.new(self, vcs: 'hg')
      end

      def gitaly_blob_client
        @gitaly_blob_client ||= Gitlab::GitalyClient::BlobService.new(self, vcs: 'hg')
      end
    end
  end
end
